"""Console script for Media Hoard CLI."""

import subprocess
import sys
import tempfile
from pathlib import Path

import click

from media_hoard_cli import hoard


@click.group()
def main():
    """Empty click anchor function."""


@click.command()
@click.option('--cfg-file',
              default="$HOME/.config/media_hoard/config.yaml",
              help="path to config file")
@click.option('--part-file', default=None, help="path to parts file")
@click.option('--upload-dir', help="overide cfg upload-dir")
@click.argument('title')
@click.argument('src_file')
def add(cfg_file, part_file, upload_dir, title, src_file):
    """Add a new file from local host.

    TITLE: User friendly title
    SRC_FILE: path to item to add (path/to/file.pdf)
    """
    with tempfile.TemporaryDirectory() as dir_temp:
        item_id = hoard.get_id()
        dir_stage = dir_temp + '/' + item_id
        Path(dir_stage).mkdir()

        try:
            parts = hoard.parse_parts_file(part_file)

        except FileNotFoundError:
            parts = []

        try:
            cfg = hoard.parse_config_file(cfg_file, upload_dir)

            item = hoard.new_item(src_file, dir_stage, parts, title)

        except FileNotFoundError as exp:
            raise click.ClickException(exp)

        subprocess.run(['rsync', '-r', dir_stage, upload_dir + '/'],
                       check=True)

        print(hoard.render_item(item, item_id, cfg['item_url']))

    return 0


main.add_command(add)

if __name__ == "__main__":
    sys.exit(main())  # pylint: disable=no-value-for-parameter



#!/usr/bin/env python
"""Tests for `media_hoard_cli` package."""
# pylint: disable=redefined-outer-name
from click.testing import CliRunner

from media_hoard_cli import cli, hoard

ITEM_NID = 'YKIKuCiQAl'


def test_item_init():
    """Test Item init."""
    item = hoard.Item(title="Simple PDF File", doc_end_pg=2, pdf_end_pg=2)

    assert item.title == "Simple PDF File"
    assert item.sub_title == ""
    assert item.pdf_start_pg == 1
    assert item.pdf_end_pg == 2
    assert item.doc_start_pg == 1
    assert item.doc_end_pg == 2
    assert item.name == "simple_pdf_file.pdf"
    assert item.pages == (0, 1)
    assert len(item.children) == 0

    args = {
        'title': "Chapter 1",
        'sub_title': "Detailed Nonsense",
        'doc_start_pg': 12,
        'doc_end_pg': 14,
        'pdf_start_pg': 14,
        'pdf_end_pg': 16
    }

    item = hoard.Item(**args)

    assert item.sub_title == "Detailed Nonsense"
    assert item.doc_start_pg == 12
    assert item.doc_end_pg == 14
    assert item.pdf_start_pg == 14
    assert item.pdf_end_pg == 16


def test_cli_missing_add_file():
    """Test exception text on missing input file."""
    runner = CliRunner()

    result = runner.invoke(cli.main, [
        'add', '--cfg-file', 'tests/fixtures/config.yaml',
        'Input File Not Found', '/no_such_file.pdf'
    ])

    assert result.exit_code == 1
    assert "/no_such_file.pdf" in result.output


def test_cli_add_pdf(mocker, tmp_path):
    """Test adding a single file."""
    mocker.patch('media_hoard_cli.hoard.nanoid.generate',
                 return_value=ITEM_NID)
    runner = CliRunner()
    result = runner.invoke(cli.main, [
        'add', '--cfg-file', 'tests/fixtures/config.yaml', '--upload-dir',
        tmp_path, 'Basic Test File', 'tests/fixtures/file.pdf'
    ])

    assert result.exit_code == 0
    assert (tmp_path / ITEM_NID / 'basic_test_file.pdf').is_file()

    expected = """Basic Test File


- http://localhost/YKIKuCiQAl/basic_test_file.pdf

"""

    assert result.output == expected


def test_cli_add_pdf_parts(mocker, tmp_path):
    """Test adding a pdf file and parts."""
    mocker.patch('media_hoard_cli.hoard.nanoid.generate',
                 return_value=ITEM_NID)
    runner = CliRunner()
    result = runner.invoke(cli.main, [
        'add', '--cfg-file', 'tests/fixtures/config.yaml', '--part-file',
        'tests/fixtures/file_parts.csv', '--upload-dir', tmp_path,
        'Basic Test File', 'tests/fixtures/file_with_parts.pdf'
    ])

    print(result.output)
    assert result.exit_code == 0
    assert (tmp_path / ITEM_NID / 'introduction.pdf').is_file()

    expected = """Basic Test File


- http://localhost/YKIKuCiQAl/basic_test_file.pdf

Introduction


pages: 1-2

- http://localhost/YKIKuCiQAl/introduction.pdf

Chapter 1


pages: 3-6

- http://localhost/YKIKuCiQAl/chapter_1.pdf

Chapter 3
Nonsense, more nonsense, and silliness

pages: 9-11

- http://localhost/YKIKuCiQAl/chapter_3.pdf

"""

    assert result.output == expected



"""Main module."""

import csv
import shutil
from dataclasses import asdict, dataclass, field
from pathlib import Path
from string import Template

import nanoid
import yaml
from PyPDF4 import PdfFileReader, PdfFileWriter

ITEM_TEMPLATE = """$title
$sub_title

- $item_url
"""

ITEM_PART_TEMPLATE = """
$title
$sub_title

pages: $doc_start_pg-$doc_end_pg

- $item_url
"""


@dataclass
class Item:  # pylint: disable=too-many-instance-attributes
    """Class representing an Item and optional children."""

    title: str
    doc_end_pg: int
    pdf_end_pg: int

    children: list = field(default_factory=list)
    doc_start_pg: int = field(default=1)
    pdf_start_pg: int = field(default=1)
    sub_title: str = field(default="")

    name: str = field(init=False)
    pages: tuple = field(init=False)

    def __post_init__(self):
        """Initilize dynamic members."""
        slug = self.title.lower().replace(' ', '_')

        self.doc_end_pg = int(self.doc_end_pg)
        self.doc_start_pg = int(self.doc_start_pg)
        self.pdf_end_pg = int(self.pdf_end_pg)
        self.pdf_start_pg = int(self.pdf_start_pg)

        self.name = f'{slug}.pdf'
        self.pages = tuple(range(self.pdf_start_pg - 1, self.pdf_end_pg))

    def as_dict(self):
        """Return instance fields as a dictionary."""
        return asdict(self)


def _add_pdf(src, dir_stage, title):

    with Path(src).open('rb') as fd_in:
        reader = PdfFileReader(fd_in)

        end_page = reader.numPages

    item = Item(title=title, doc_end_pg=end_page, pdf_end_pg=end_page)

    shutil.copy(src, (Path(dir_stage) / item.name))

    return item


def _add_pdf_part(src, dir_stage, part):

    with Path(src).open('rb') as fd_in:
        reader = PdfFileReader(fd_in)
        writer = PdfFileWriter()

        for page in part.pages:
            writer.addPage(reader.getPage(page))

        with (Path(dir_stage) / part.name).open('wb') as fd_out:
            writer.write(fd_out)

    return part


def get_id():
    """Return a nanoid string."""
    return nanoid.generate(size=10)


def new_item(src, dir_stage, parts, title):
    """Create a new Item object.

    :param src: path to file to add
    :type src: str
    :param dir_stage: path to staging directory
    :dir_stage: str
    :param title: User friendly title for item
    :type title: str

    :return: Item
    :rtype: Item
    """
    item = _add_pdf(src, dir_stage, title)

    for part in parts:
        item.children.append(_add_pdf_part(src, dir_stage, part))

    return item


def parse_config_file(path, upload_dir):
    """Return config from yaml file at path."""
    with Path(path).open(encoding='utf-8') as fd_in:
        cfg = yaml.safe_load(fd_in)
        cfg['upload_dir'] = cfg['upload_dir'] if not upload_dir else upload_dir

        if cfg['upload_dir'][-1] != '/':
            cfg['upload_dir'] = cfg['upload_dir'] + '/'

        return cfg


def parse_parts_file(path):
    """Parse parts file and return a list of Items."""
    parts = []

    try:
        with Path(path).open(encoding='utf-8', newline='') as fd_in:
            reader = csv.DictReader(fd_in)

            for row in reader:
                parts.append(Item(**row))

    except TypeError:
        pass

    return parts


def render_item(item, item_id, item_url_str):
    """Render an item and optional children to a text string."""
    fields = item.as_dict()
    fields['item_url'] = Template(item_url_str).substitute(nid=item_id,
                                                           name=item.name)

    out = Template(ITEM_TEMPLATE).substitute(**fields)

    part_template = Template(ITEM_PART_TEMPLATE)
    item_url_template = Template(item_url_str)

    for child in item.children:
        fields = child.as_dict()
        fields['item_url'] = item_url_template.substitute(nid=item_id,
                                                          name=child.name)
        out += part_template.substitute(**fields)

    return out



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['media_hoard_cli', 'tests']

package_data = \
{'': ['*'], 'media_hoard_cli': ['templates/*'], 'tests': ['fixtures/*']}

install_requires = \
['PyPDF4>=1.27.0,<2.0.0', 'PyYAML>=6.0,<7.0', 'click', 'nanoid>=2.0.0,<3.0.0']

entry_points = \
{'console_scripts': ['media_hoard = media_hoard_cli.cli:main']}

setup_kwargs = {
    'name': 'media-hoard-cli',
    'version': '0.2.0',
    'description': 'Top-level package for Media Hoard CLI.',
    'long_description': '\nMedia Hoard CLI\n\n* Free software: MIT License\n\nFeatures\n--------\n\n* TODO\n\n\nCredits\n-------\n\nThis package was created with Cookiecutter_ and the `midwatch/cc-py3-pkg`_ project template.\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`midwatch/cc-py3-pkg`: https://github.com/midwatch/cc-py3-pkg\n',
    'author': 'Justin Stout',
    'author_email': 'midwatch@jstout.us',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/midwatch/media_hoard_cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
